      SUBROUTINE rk4(y,dydx,h,deltaX,iprof)
      implicit double precision (a-h,o-z)
c
c  Brian Dushaw
c  Applied Physics Laboratory
c  University of Washington
c  Seattle, WA 98105
c  (206) 543-1300
c  dushaw@apl.washington.edu
c
c  EIGENRAY Version 3.4
c  September-December 1998
c  Completely revised November 2012
c
c  5-6th order Runge-Kutta-Merson, i.e., 5th order with a natural correction to 6th order.
c  The integration is constructed in four steps, with an error estimate at the end.  
c  The error estimate becomes a natural correction to the integration to the next oder.
c
c  The derivatives are all inlined here rather than subroutined because 
c  this seems to make the code execute faster.
c
      DOUBLE PRECISION dydx(4),y(4),err(4)
      DOUBLE PRECISION dy1(4),dy2(4),dy3(4),dy4(4)
      DOUBLE PRECISION yt1(4),yt2(4),yt3(4),yt4(4),yt5(4)
      double precision deltaX
      integer iprof
      parameter(othirty=1.d0/30.d0,scl=0.5d0,eight=8.d0,xnine=9.d0)

      include 'nssp.h'
      include 'table.h'
      include 'constants.h'

1     h2=h*half 
      h22=two*h
      h3=h*third
      h23=two*h3
      h32=three*h2
      h8=h*0.125d0
      h38=h*0.375d0
      h6=h*sixth
      h30=h*othirty
c y_1
      do 11 i=1,3
        yt1(i)=y(i)+h3*dydx(i)
11    continue
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      z=yt1(3)
      if(z.gt.XMAXDEPTH) then
        y(3)=z
        return
      endif
      ii=int(xinv_depth_inc*z)
      if(z.lt.zero)ii=-1
      dz=z-tab(ii)
      dz2=dz*half
      dz3=dz*third

c These are the cubic spline coefficients; ii gives the appropriate interval.
c See the cubic spline routines cubspl and ppvalu for what is going on here.
       x1=etab(1,ii,iprof)
       x2=etab(2,ii,iprof)
       x3=etab(3,ii,iprof)
       x4=etab(4,ii,iprof)
c Get sound speed.
       c = ((x4*dz3 + x3)*dz2+x2)*dz+x1

c Get sound speed gradient.
       dcdz = (x4*dz2 + x3)*dz+x2

c Get horizontal gradients; same procedure but increment iprof by one.
       x1=etab(1,ii,iprof+1)
       x2=etab(2,ii,iprof+1)
       x3=etab(3,ii,iprof+1)
       x4=etab(4,ii,iprof+1)
       cp = ((x4*dz3 + x3)*dz2+x2)*dz+x1
       dcdzp = (x4*dz2 + x3)*dz+x2
       C1 = (cp-c)*xinvr(iprof)
       C2 = (dcdzp-dcdz)*xinvr(iprof)

c Final sound speed and sound speed gradient.
c Negative dcdz - positive z down.
       c = c + deltaX*C1
       dcdz = -dcdz - deltaX*C2

       c_inv=one/c
       y_inv=one/yt1(1)
       dy1(3)=  yt1(2)*y_inv
c       q1= (dcdz - C1*yt1(2)*y_inv)*c_inv
       q1= (dcdz - C1*dy1(3))*c_inv

       dy1(1)= -yt1(2)*q1
       dy1(2)=  yt1(1)*q1
c       dy1(4)=  c_inv*y_inv
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c y_2
      do 12 i=1,3
        yt2(i)=y(i)+h6*(dydx(i)+dy1(i))
12    continue
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      z=yt2(3)
      if(z.gt.XMAXDEPTH) then
        y(3)=z
        return
      endif
      ii=int(xinv_depth_inc*z)
      if(z.lt.zero)ii=-1
      dz=z-tab(ii)
      dz2=dz*half
      dz3=dz*third

       x1=etab(1,ii,iprof)
       x2=etab(2,ii,iprof)
       x3=etab(3,ii,iprof)
       x4=etab(4,ii,iprof)
c get sound speed
       c = ((x4*dz3 + x3)*dz2+x2)*dz+x1

c get sound speed gradient
       dcdz = (x4*dz2 + x3)*dz+x2

c get horizontal gradients
       x1=etab(1,ii,iprof+1)
       x2=etab(2,ii,iprof+1)
       x3=etab(3,ii,iprof+1)
       x4=etab(4,ii,iprof+1)
       cp = ((x4*dz3 + x3)*dz2+x2)*dz+x1
       dcdzp = (x4*dz2 + x3)*dz+x2
       C1 = (cp-c)*xinvr(iprof)
       C2 = (dcdzp-dcdz)*xinvr(iprof)

c final sound speed, sound speed gradient
       c = c + deltaX*C1
       dcdz = -dcdz - deltaX*C2

       c_inv=one/c
       y_inv=one/yt2(1)
       dy2(3)=  yt2(2)*y_inv
c       q1= (dcdz - C1*yt1(2)*y_inv)*c_inv
       q1= (dcdz - C1*dy2(3))*c_inv

       dy2(1)= -yt2(2)*q1
       dy2(2)=  yt2(1)*q1
       dy2(4)=  c_inv*y_inv

c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c y_3
      do 13 i=1,3
        yt3(i)=y(i)+h8*dydx(i)+h38*dy2(i)
13    continue
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      z=yt3(3)
      if(z.gt.XMAXDEPTH) then
        y(3)=z
        return
      endif
      ii=int(xinv_depth_inc*z)
      if(z.lt.zero)ii=-1
      dz=z-tab(ii)
      dz2=dz*half
      dz3=dz*third

       x1=etab(1,ii,iprof)
       x2=etab(2,ii,iprof)
       x3=etab(3,ii,iprof)
       x4=etab(4,ii,iprof)
c get sound speed
       c = ((x4*dz3 + x3)*dz2+x2)*dz+x1

c get sound speed gradient
       dcdz = (x4*dz2 + x3)*dz+x2

c get horizontal gradients
       x1=etab(1,ii,iprof+1)
       x2=etab(2,ii,iprof+1)
       x3=etab(3,ii,iprof+1)
       x4=etab(4,ii,iprof+1)
       cp = ((x4*dz3 + x3)*dz2+x2)*dz+x1
       dcdzp = (x4*dz2 + x3)*dz+x2
       C1 = (cp-c)*xinvr(iprof)
       C2 = (dcdzp-dcdz)*xinvr(iprof)

c final sound speed, sound speed gradient
       c = c + deltaX*C1
       dcdz = -dcdz - deltaX*C2

       c_inv=one/c
       y_inv=one/yt3(1)
       dy3(3)=  yt3(2)*y_inv
c       q1= (dcdz - C1*yt1(2)*y_inv)*c_inv
       q1= (dcdz - C1*dy3(3))*c_inv

       dy3(1)= -yt3(2)*q1
       dy3(2)=  yt3(1)*q1
       dy3(4)=  c_inv*y_inv
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c y_4
      do 14 i=1,3
        yt4(i)=y(i)+h2*dydx(i)-h32*dy2(i)+h22*dy3(i)
14    continue
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      z=yt4(3)
      if(z.gt.XMAXDEPTH) then
        y(3)=z
        return
      endif
      ii=int(xinv_depth_inc*z)
      if(z.lt.zero)ii=-1
      dz=z-tab(ii)
      dz2=dz*half
      dz3=dz*third

       x1=etab(1,ii,iprof)
       x2=etab(2,ii,iprof)
       x3=etab(3,ii,iprof)
       x4=etab(4,ii,iprof)
c get sound speed
       c = ((x4*dz3 + x3)*dz2+x2)*dz+x1

c get sound speed gradient
       dcdz = (x4*dz2 + x3)*dz+x2

c get horizontal gradients
       x1=etab(1,ii,iprof+1)
       x2=etab(2,ii,iprof+1)
       x3=etab(3,ii,iprof+1)
       x4=etab(4,ii,iprof+1)
       cp = ((x4*dz3 + x3)*dz2+x2)*dz+x1
       dcdzp = (x4*dz2 + x3)*dz+x2
       C1 = (cp-c)*xinvr(iprof)
       C2 = (dcdzp-dcdz)*xinvr(iprof)

c final sound speed, sound speed gradient
       c = c + deltaX*C1
       dcdz = -dcdz - deltaX*C2

c Comment out those things that aren't needed until the error criteria is acceptable.
c (Save a division!)
c      c_inv=one/c
       y_inv=one/yt4(1)
c       q1= (dcdz - C1*yt1(2)*y_inv)*c_inv

c       dy4(1)= -yt4(2)*q1
c       dy4(2)=  yt4(1)*q1
       dy4(3)=  yt4(2)*y_inv
c       dy4(4)=  c_inv*y_inv
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c  The error for the sine or depth components seem to be the most sensitive test.
c  We'll use depth here.
       err(3)=(two*dydx(3)-xnine*dy2(3)+eight*dy3(3)-dy4(3))*h30
       if ((dabs(err(3)).gt.epsilon).and.(h.gt.one)) then
c  Step sizes should always be greater than 1 meter, seems to me.
c  If it gets to be less then 1 meter, we are asking too much.
             h=h*scl
             goto 1
       endif
c     Out of epsilon loop:  either the error criteria has been
c     achieved, or cant pass error criteria with h > 1 m.

       c_inv=one/c
c       q1= (dcdz - C1*yt1(2)*y_inv)*c_inv
       q1= (dcdz - C1*dy4(3))*c_inv

       dy4(1)= -yt4(2)*q1
       dy4(2)=  yt4(1)*q1
c       dy4(3)=  yt4(2)*y_inv
       dy4(4)=  c_inv*y_inv
       err(1)=(two*dydx(1)-xnine*dy2(1)+eight*dy3(1)-dy4(1))*h30
       err(2)=(two*dydx(2)-xnine*dy2(2)+eight*dy3(2)-dy4(2))*h30
c       err(3)=(two*dydx(3)-xnine*dy2(3)+eight*dy3(3)-dy4(3))*h30
       err(4)=(two*dydx(4)-xnine*dy2(4)+eight*dy3(4)-dy4(4))*h30

       do i=1,4
c Subtract err to get integration to O(h^6) accuracy 
c This error is a real, honest correction to get to the next order.
        yt5(i)=y(i)+h23*dy3(i)+h6*(dy4(i)+dydx(i))
        y(i)=yt5(i)-err(i)
c         y(i)=y(i) + h23*dy3(i)+h6*(dy4(i)+dydx(i)) - err(i)
       enddo

      return

      END
